#!/usr/bin/env python

import sys
import os

scripts = (
    ('CRFsuite 0.12', './bench_crfsuite.py'),
    ('CRFsuite 0.11', './bench_crfsuite-0.11.py'),
    ('Wapiti v1.1.3', './bench_wapiti.py'),
    ('sgd 1.3', './bench_crfsgd.py'),
    ('CRF++ 0.54', './bench_crfpp.py'),
    ('MALLET 2.0.6', './bench_mallet.py'),
)

fields = (
    ('# Features', 'num_features'),
    ('Time', 'time'),
    ('# Iters', 'iterations'),
    ('Update', 'update'),
    ('Loss', 'loss'),
    ('Log', 'log'),
)

def number(x):
    y = ''
    p = x.find('.')
    if p == -1:
        p = len(x)
    for i in range(p):
        if i % 3 == 0 and i != 0:
            y = ' ' + y
        y = x[p-i-1] + y
    return y + x[p:]

def read():
    R = {}
    for name, script in scripts:
        fi = os.popen(script, 'r')
        R[name] = eval(fi.read())
    return R

def output_update(fo, R):
    for name, script in scripts:
        for param, result in R[name].iteritems():
            fo.write('%s\t%s\t%f\n' % (name, param, result.get('update', 0.)))

def output_table(fo, R):
    for name, script in scripts:
        for param, result in R[name].iteritems():
            fo.write('<row>\n')
            fo.write('<entry>%s</entry>\n' % name)
            fo.write('<entry>%s</entry>\n' % param)
            fo.write('<entry></entry>\n')
            fo.write('<entry>%s</entry>\n' % number('%d' % result['num_features']))
            fo.write('<entry>%s</entry>\n' % number('%.1f' % result['time']))
            fo.write('<entry>%s</entry>\n' % number('%d' % result['iterations']))
            fo.write('<entry>%s</entry>\n' % number('%.1f' % result['update']))
            fo.write('<entry>%s</entry>\n' % number('%.1f' % result['loss']))
            fo.write('<entry>%.3f</entry>\n' % (100. * result['accuracy']))
            fo.write('<entry><ulink url="%s">Log</ulink></entry>\n' % result['logfile'])
            fo.write('</row>\n')
            fo.write('\n')


if __name__ == '__main__':
    R = read()
    output_table(sys.stdout, R)



#!/usr/bin/env python

import crfsuite
import sys

def instances(fi):
    xseq = crfsuite.ItemSequence()
    
    for line in fi:
        line = line.strip('\n')
        if not line:
        	# An empty line presents an end of a sequence.
            yield xseq
            xseq = crfsuite.ItemSequence()
            continue

		# Split the line with TAB characters.
        fields = line.split('\t')
        item = crfsuite.Item()
        for field in fields[1:]:
            p = field.rfind(':')
            if p == -1:
            	# Unweighted (weight=1) attribute.
                item.append(crfsuite.Attribute(field))
            else:
            	# Weighted attribute
                item.append(crfsuite.Attribute(field[:p], float(field[p+1:])))

        # Append the item to the item sequence.
        xseq.append(item)

if __name__ == '__main__':
    fi = sys.stdin
    fo = sys.stdout

	# Create a tagger object.
    tagger = crfsuite.Tagger()
    
    # Load the model to the tagger.
    tagger.open(sys.argv[1])

    for xseq in instances(fi):
    	# Tag the sequence.
        tagger.set(xseq)
        # Obtain the label sequence predicted by the tagger.
        yseq = tagger.viterbi()
        # Output the probability of the predicted label sequence.
        print tagger.probability(yseq)
        for t, y in enumerate(yseq):
        	# Output the predicted labels with their marginal probabilities.
            print '%s:%f' % (y, tagger.marginal(y, t))
        print



#!/usr/bin/env python

import sys

fi = sys.stdin
fo = sys.stdout

for line in fi:
    line = line.strip('\n')
    if not line:
        fo.write('\n')

    fields = line.split('\t')
    fo.write('%s %s\n' % (' '.join(fields[1:]), fields[0]))

    



#!/usr/bin/env python

import sys
import os
import string
from bench import *

CRFSUITE='/home/okazaki/projects/crfsuite/frontend/crfsuite'
OUTDIR='crfsuite/'

training_patterns = (
    ('num_features', r'^Number of features: (\d+)', 1, int, last),
    ('time', r'^Total seconds required for training: ([\d.]+)', 1, float, last),
    ('iterations', r'^\*\*\*\*\* (Iteration|Epoch) #(\d+)', 2, int, last),
    ('update', r'^Seconds required for this iteration: ([\d.]+)', 1, float, min),
    ('loss', r'^Loss: ([-\d.]+)', 1, float, last),
)

tagging_patterns = (
    ('accuracy', r'^Item accuracy: \d+ / \d+ \(([\d.]+)\)', 1, float, last),
)

params = {
    'lbfgs-sparse': '-a lbfgs -p feature.possible_states=0 -p feature.possible_transitions=0',
    'lbfgs-dense': '-a lbfgs -p feature.possible_states=1 -p feature.possible_transitions=1',
    'l2sgd-sparse': '-a l2sgd -p feature.possible_states=0 -p feature.possible_transitions=0',
    'l2sgd-dense': '-a l2sgd -p feature.possible_states=1 -p feature.possible_transitions=1',
    'ap-sparse': '-a ap -p feature.possible_states=0 -p feature.possible_transitions=0 -p max_iterations=50',
    'ap-dense': '-a ap -p feature.possible_states=1 -p feature.possible_transitions=1 -p max_iterations=50',
}

if __name__ == '_main__':
    print analyze_log(sys.stdin, training_patterns)

if __name__ == '__main__':
    fe = sys.stderr

    R = {}
    for name, param in params.iteritems():
        model = OUTDIR + name + '.model'
        trlog = OUTDIR + name + '.tr.log'
        trtxt = LOGDIR + 'crfsuite-' + name + '.txt'
        tglog = OUTDIR + name + '.tg.log'

        s = string.Template(
            '$crfsuite learn $param -m $model train.crfsuite > $trlog'
            )
        cmd = s.substitute(
            crfsuite=CRFSUITE,
            param=param,
            model=model,
            trlog=trlog
            )

        fe.write(cmd)
        fe.write('\n')
        #os.system(cmd)

        fo = open(trtxt, 'w')
        fo.write('$ %s\n' % cmd)
        fo.write(open(trlog, 'r').read())

        s = string.Template(
            '$crfsuite tag -m $model -qt test.crfsuite > $tglog'
            )
        cmd = s.substitute(
            crfsuite=CRFSUITE,
            model=model,
            tglog=tglog
            )

        fe.write(cmd)
        fe.write('\n')
        #os.system(cmd)

        D = analyze_log(open(trlog), training_patterns)
        D.update(analyze_log(open(tglog), tagging_patterns))
        D['logfile'] = trtxt
        R[name] = D

    print repr(R)



#!/usr/bin/env python

import sys
import os
import string
from bench import *

CRFSGD='/home/okazaki/install/sgd-1.3/crf/crfsgd'
OUTDIR='crfsgd/'

training_patterns = (
    ('num_features', r'features: (\d+)', 1, int, last),
    ('time', r'^Done!  ([\d.]+)', 1, float, last),
    ('iterations', r'^\[Epoch (\d+)\]', 1, int, last),
    ('update', r'^\[Epoch \d+\][^a-z]+wnorm:[^a-z]+total time: ([\d.]+) seconds$', 1, float, diffmin),
    ('loss', r'loss: ([\d.]+)', 1, float, last),
)

tagging_patterns = (
    ('accuracy', r'^Item accuracy: ([\d.]+)', 1, float, last),
)

params = {
    'default': "-f 1 -r 100 -e ''",
}

if __name__ == '__main__':
    fe = sys.stderr

    R = {}
    for name, param in params.iteritems():
        model = OUTDIR + name + '.model'
        trlog = OUTDIR + name + '.tr.log'
        trtxt = LOGDIR + 'crfsgd-' + name + '.txt'
        tglog = OUTDIR + name + '.tg.log'

        s = string.Template(
            '$crfsgd $param $model template.crfpp train.txt > $trlog'
            )
        cmd = s.substitute(
            crfsgd=CRFSGD,
            param=param,
            model=model,
            trlog=trlog
            )

        fe.write(cmd)
        fe.write('\n')
        #os.system(cmd)

        fo = open(trtxt, 'w')
        fo.write('$ %s\n' % cmd)
        fo.write(open(trlog, 'r').read())

        s = string.Template(
            '$crfsgd -t $model test.txt | ./accuracy.py > $tglog'
            )
        cmd = s.substitute(
            crfsgd=CRFSGD,
            model=model,
            tglog=tglog
            )

        fe.write(cmd)
        fe.write('\n')
        #os.system(cmd)

        D = analyze_log(open(trlog), training_patterns)
        D.update(analyze_log(open(tglog), tagging_patterns))
        D['logfile'] = trtxt
        R[name] = D

    print repr(R)

if __name__ == '_main__':
    print analyze_log(sys.stdin, training_patterns)



#!/usr/bin/env python

import sys

if __name__ == '__main__':
    fi = sys.stdin
    fo = sys.stdout
    n = 0
    m = 0

    for line in fi:
        line = line.strip()
        if line:
            fields = line.split()
            if len(fields) >= 2:
                if fields[-1] == fields[-2]:
                    m += 1
                n += 1

    print 'Item accuracy: %f' % (m / float(n))



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.





from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_crfsuite', [dirname(__file__)])
        except ImportError:
            import _crfsuite
            return _crfsuite
        if fp is not None:
            try:
                _mod = imp.load_module('_crfsuite', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _crfsuite = swig_import_helper()
    del swig_import_helper
else:
    import _crfsuite
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0


try:
    import weakref
    weakref_proxy = weakref.proxy
except:
    weakref_proxy = lambda x: x


class SwigPyIterator(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SwigPyIterator, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SwigPyIterator, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _crfsuite.delete_SwigPyIterator
    __del__ = lambda self : None;
    def value(self): return _crfsuite.SwigPyIterator_value(self)
    def incr(self, n=1): return _crfsuite.SwigPyIterator_incr(self, n)
    def decr(self, n=1): return _crfsuite.SwigPyIterator_decr(self, n)
    def distance(self, *args): return _crfsuite.SwigPyIterator_distance(self, *args)
    def equal(self, *args): return _crfsuite.SwigPyIterator_equal(self, *args)
    def copy(self): return _crfsuite.SwigPyIterator_copy(self)
    def next(self): return _crfsuite.SwigPyIterator_next(self)
    def __next__(self): return _crfsuite.SwigPyIterator___next__(self)
    def previous(self): return _crfsuite.SwigPyIterator_previous(self)
    def advance(self, *args): return _crfsuite.SwigPyIterator_advance(self, *args)
    def __eq__(self, *args): return _crfsuite.SwigPyIterator___eq__(self, *args)
    def __ne__(self, *args): return _crfsuite.SwigPyIterator___ne__(self, *args)
    def __iadd__(self, *args): return _crfsuite.SwigPyIterator___iadd__(self, *args)
    def __isub__(self, *args): return _crfsuite.SwigPyIterator___isub__(self, *args)
    def __add__(self, *args): return _crfsuite.SwigPyIterator___add__(self, *args)
    def __sub__(self, *args): return _crfsuite.SwigPyIterator___sub__(self, *args)
    def __iter__(self): return self
SwigPyIterator_swigregister = _crfsuite.SwigPyIterator_swigregister
SwigPyIterator_swigregister(SwigPyIterator)

class Attribute(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Attribute, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Attribute, name)
    __repr__ = _swig_repr
    __swig_setmethods__["attr"] = _crfsuite.Attribute_attr_set
    __swig_getmethods__["attr"] = _crfsuite.Attribute_attr_get
    if _newclass:attr = _swig_property(_crfsuite.Attribute_attr_get, _crfsuite.Attribute_attr_set)
    __swig_setmethods__["value"] = _crfsuite.Attribute_value_set
    __swig_getmethods__["value"] = _crfsuite.Attribute_value_get
    if _newclass:value = _swig_property(_crfsuite.Attribute_value_get, _crfsuite.Attribute_value_set)
    def __init__(self, *args): 
        this = _crfsuite.new_Attribute(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _crfsuite.delete_Attribute
    __del__ = lambda self : None;
Attribute_swigregister = _crfsuite.Attribute_swigregister
Attribute_swigregister(Attribute)

class Trainer(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Trainer, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Trainer, name)
    __repr__ = _swig_repr
    def __init__(self): 
        if self.__class__ == Trainer:
            _self = None
        else:
            _self = self
        this = _crfsuite.new_Trainer(_self, )
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _crfsuite.delete_Trainer
    __del__ = lambda self : None;
    def clear(self): return _crfsuite.Trainer_clear(self)
    def append(self, *args): return _crfsuite.Trainer_append(self, *args)
    def select(self, *args): return _crfsuite.Trainer_select(self, *args)
    def train(self, *args): return _crfsuite.Trainer_train(self, *args)
    def params(self): return _crfsuite.Trainer_params(self)
    def set(self, *args): return _crfsuite.Trainer_set(self, *args)
    def get(self, *args): return _crfsuite.Trainer_get(self, *args)
    def help(self, *args): return _crfsuite.Trainer_help(self, *args)
    def message(self, *args): return _crfsuite.Trainer_message(self, *args)
    def __disown__(self):
        self.this.disown()
        _crfsuite.disown_Trainer(self)
        return weakref_proxy(self)
Trainer_swigregister = _crfsuite.Trainer_swigregister
Trainer_swigregister(Trainer)

class Tagger(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Tagger, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Tagger, name)
    __repr__ = _swig_repr
    def __init__(self): 
        this = _crfsuite.new_Tagger()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _crfsuite.delete_Tagger
    __del__ = lambda self : None;
    def open(self, *args): return _crfsuite.Tagger_open(self, *args)
    def close(self): return _crfsuite.Tagger_close(self)
    def labels(self): return _crfsuite.Tagger_labels(self)
    def tag(self, *args): return _crfsuite.Tagger_tag(self, *args)
    def set(self, *args): return _crfsuite.Tagger_set(self, *args)
    def viterbi(self): return _crfsuite.Tagger_viterbi(self)
    def probability(self, *args): return _crfsuite.Tagger_probability(self, *args)
    def marginal(self, *args): return _crfsuite.Tagger_marginal(self, *args)
Tagger_swigregister = _crfsuite.Tagger_swigregister
Tagger_swigregister(Tagger)


def version():
  return _crfsuite.version()
version = _crfsuite.version
class Item(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Item, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Item, name)
    __repr__ = _swig_repr
    def iterator(self): return _crfsuite.Item_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _crfsuite.Item___nonzero__(self)
    def __bool__(self): return _crfsuite.Item___bool__(self)
    def __len__(self): return _crfsuite.Item___len__(self)
    def pop(self): return _crfsuite.Item_pop(self)
    def __getslice__(self, *args): return _crfsuite.Item___getslice__(self, *args)
    def __setslice__(self, *args): return _crfsuite.Item___setslice__(self, *args)
    def __delslice__(self, *args): return _crfsuite.Item___delslice__(self, *args)
    def __delitem__(self, *args): return _crfsuite.Item___delitem__(self, *args)
    def __getitem__(self, *args): return _crfsuite.Item___getitem__(self, *args)
    def __setitem__(self, *args): return _crfsuite.Item___setitem__(self, *args)
    def append(self, *args): return _crfsuite.Item_append(self, *args)
    def empty(self): return _crfsuite.Item_empty(self)
    def size(self): return _crfsuite.Item_size(self)
    def clear(self): return _crfsuite.Item_clear(self)
    def swap(self, *args): return _crfsuite.Item_swap(self, *args)
    def get_allocator(self): return _crfsuite.Item_get_allocator(self)
    def begin(self): return _crfsuite.Item_begin(self)
    def end(self): return _crfsuite.Item_end(self)
    def rbegin(self): return _crfsuite.Item_rbegin(self)
    def rend(self): return _crfsuite.Item_rend(self)
    def pop_back(self): return _crfsuite.Item_pop_back(self)
    def erase(self, *args): return _crfsuite.Item_erase(self, *args)
    def __init__(self, *args): 
        this = _crfsuite.new_Item(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _crfsuite.Item_push_back(self, *args)
    def front(self): return _crfsuite.Item_front(self)
    def back(self): return _crfsuite.Item_back(self)
    def assign(self, *args): return _crfsuite.Item_assign(self, *args)
    def resize(self, *args): return _crfsuite.Item_resize(self, *args)
    def insert(self, *args): return _crfsuite.Item_insert(self, *args)
    def reserve(self, *args): return _crfsuite.Item_reserve(self, *args)
    def capacity(self): return _crfsuite.Item_capacity(self)
    __swig_destroy__ = _crfsuite.delete_Item
    __del__ = lambda self : None;
Item_swigregister = _crfsuite.Item_swigregister
Item_swigregister(Item)

class ItemSequence(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, ItemSequence, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, ItemSequence, name)
    __repr__ = _swig_repr
    def iterator(self): return _crfsuite.ItemSequence_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _crfsuite.ItemSequence___nonzero__(self)
    def __bool__(self): return _crfsuite.ItemSequence___bool__(self)
    def __len__(self): return _crfsuite.ItemSequence___len__(self)
    def pop(self): return _crfsuite.ItemSequence_pop(self)
    def __getslice__(self, *args): return _crfsuite.ItemSequence___getslice__(self, *args)
    def __setslice__(self, *args): return _crfsuite.ItemSequence___setslice__(self, *args)
    def __delslice__(self, *args): return _crfsuite.ItemSequence___delslice__(self, *args)
    def __delitem__(self, *args): return _crfsuite.ItemSequence___delitem__(self, *args)
    def __getitem__(self, *args): return _crfsuite.ItemSequence___getitem__(self, *args)
    def __setitem__(self, *args): return _crfsuite.ItemSequence___setitem__(self, *args)
    def append(self, *args): return _crfsuite.ItemSequence_append(self, *args)
    def empty(self): return _crfsuite.ItemSequence_empty(self)
    def size(self): return _crfsuite.ItemSequence_size(self)
    def clear(self): return _crfsuite.ItemSequence_clear(self)
    def swap(self, *args): return _crfsuite.ItemSequence_swap(self, *args)
    def get_allocator(self): return _crfsuite.ItemSequence_get_allocator(self)
    def begin(self): return _crfsuite.ItemSequence_begin(self)
    def end(self): return _crfsuite.ItemSequence_end(self)
    def rbegin(self): return _crfsuite.ItemSequence_rbegin(self)
    def rend(self): return _crfsuite.ItemSequence_rend(self)
    def pop_back(self): return _crfsuite.ItemSequence_pop_back(self)
    def erase(self, *args): return _crfsuite.ItemSequence_erase(self, *args)
    def __init__(self, *args): 
        this = _crfsuite.new_ItemSequence(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _crfsuite.ItemSequence_push_back(self, *args)
    def front(self): return _crfsuite.ItemSequence_front(self)
    def back(self): return _crfsuite.ItemSequence_back(self)
    def assign(self, *args): return _crfsuite.ItemSequence_assign(self, *args)
    def resize(self, *args): return _crfsuite.ItemSequence_resize(self, *args)
    def insert(self, *args): return _crfsuite.ItemSequence_insert(self, *args)
    def reserve(self, *args): return _crfsuite.ItemSequence_reserve(self, *args)
    def capacity(self): return _crfsuite.ItemSequence_capacity(self)
    __swig_destroy__ = _crfsuite.delete_ItemSequence
    __del__ = lambda self : None;
ItemSequence_swigregister = _crfsuite.ItemSequence_swigregister
ItemSequence_swigregister(ItemSequence)

class StringList(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, StringList, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, StringList, name)
    __repr__ = _swig_repr
    def iterator(self): return _crfsuite.StringList_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _crfsuite.StringList___nonzero__(self)
    def __bool__(self): return _crfsuite.StringList___bool__(self)
    def __len__(self): return _crfsuite.StringList___len__(self)
    def pop(self): return _crfsuite.StringList_pop(self)
    def __getslice__(self, *args): return _crfsuite.StringList___getslice__(self, *args)
    def __setslice__(self, *args): return _crfsuite.StringList___setslice__(self, *args)
    def __delslice__(self, *args): return _crfsuite.StringList___delslice__(self, *args)
    def __delitem__(self, *args): return _crfsuite.StringList___delitem__(self, *args)
    def __getitem__(self, *args): return _crfsuite.StringList___getitem__(self, *args)
    def __setitem__(self, *args): return _crfsuite.StringList___setitem__(self, *args)
    def append(self, *args): return _crfsuite.StringList_append(self, *args)
    def empty(self): return _crfsuite.StringList_empty(self)
    def size(self): return _crfsuite.StringList_size(self)
    def clear(self): return _crfsuite.StringList_clear(self)
    def swap(self, *args): return _crfsuite.StringList_swap(self, *args)
    def get_allocator(self): return _crfsuite.StringList_get_allocator(self)
    def begin(self): return _crfsuite.StringList_begin(self)
    def end(self): return _crfsuite.StringList_end(self)
    def rbegin(self): return _crfsuite.StringList_rbegin(self)
    def rend(self): return _crfsuite.StringList_rend(self)
    def pop_back(self): return _crfsuite.StringList_pop_back(self)
    def erase(self, *args): return _crfsuite.StringList_erase(self, *args)
    def __init__(self, *args): 
        this = _crfsuite.new_StringList(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _crfsuite.StringList_push_back(self, *args)
    def front(self): return _crfsuite.StringList_front(self)
    def back(self): return _crfsuite.StringList_back(self)
    def assign(self, *args): return _crfsuite.StringList_assign(self, *args)
    def resize(self, *args): return _crfsuite.StringList_resize(self, *args)
    def insert(self, *args): return _crfsuite.StringList_insert(self, *args)
    def reserve(self, *args): return _crfsuite.StringList_reserve(self, *args)
    def capacity(self): return _crfsuite.StringList_capacity(self)
    __swig_destroy__ = _crfsuite.delete_StringList
    __del__ = lambda self : None;
StringList_swigregister = _crfsuite.StringList_swigregister
StringList_swigregister(StringList)

# This file is compatible with both classic and new-style classes.





#!/usr/bin/env python

import sys
import re

re_iteration = re.compile(r'^\*\*\*\*\* (Iteration|Epoch) #(\d+) \*\*\*\*\*')
patterns = {
    'loss': re.compile(r'^Loss: ([\d.]+)'),
    'accuracy': re.compile(r'^Item accuracy: \d+ / \d+ \(([\d.]+)\)'),
    'norm': re.compile(r'^Feature [L2-]+norm: ([\d.]+)'),
}

def read(fi):
    D = []
    for line in fi:
        line = line.strip('\n')
        m = re_iteration.match(line)
        if m is not None:
            if len(D)+1 != int(m.group(2)):
                sys.stderr.write('ERROR: sync\n')
                sys.exit(1)
            D.append({})
            continue

        if D:
            for name, pattern in patterns.iteritems():
                m = pattern.match(line)
                if m is not None:
                    D[-1][name] = float(m.group(1))
    
    return D

if __name__ == '__main__':
    fi = sys.stdin
    fo = sys.stdout

    i = 1
    D = read(fi)
    for item in D:
        fo.write('%d' % i)
        i += 1
        for name in patterns.iterkeys():
            fo.write(' %f' % item[name])
        fo.write('\n')



#!/usr/bin/env python

"""
A feature extractor for chunking.
Copyright 2010,2011 Naoaki Okazaki.
"""

# Separator of field values.
separator = ' '

# Field names of the input data.
fields = 'w pos y'

# Attribute templates.
templates = (
    (('w', -2), ),
    (('w', -1), ),
    (('w',  0), ),
    (('w',  1), ),
    (('w',  2), ),
    (('w', -1), ('w',  0)),
    (('w',  0), ('w',  1)),
    (('pos', -2), ),
    (('pos', -1), ),
    (('pos',  0), ),
    (('pos',  1), ),
    (('pos',  2), ),
    (('pos', -2), ('pos', -1)),
    (('pos', -1), ('pos',  0)),
    (('pos',  0), ('pos',  1)),
    (('pos',  1), ('pos',  2)),
    (('pos', -2), ('pos', -1), ('pos',  0)),
    (('pos', -1), ('pos',  0), ('pos',  1)),
    (('pos',  0), ('pos',  1), ('pos',  2)),
    )


import crfutils

def feature_extractor(X):
    # Apply attribute templates to obtain features (in fact, attributes)
    crfutils.apply_templates(X, templates)
    if X:
	# Append BOS and EOS features manually
        X[0]['F'].append('__BOS__')     # BOS feature
        X[-1]['F'].append('__EOS__')    # EOS feature

if __name__ == '__main__':
    crfutils.main(feature_extractor, fields=fields, sep=separator)



#!/usr/bin/env python

"""
A feature extractor for named eneity recognition (NER).
Copyright 2010,2011 Naoaki Okazaki.
"""

# Separator of field values.
separator = ' '

# Field names of the input data.
fields = 'y w pos chk'


import crfutils

def get_shape(token):
    r = ''
    for c in token:
        if c.isupper():
            r += 'U'
        elif c.islower():
            r += 'L'
        elif c.isdigit():
            r += 'D'
        elif c in ('.', ','):
            r += '.'
        elif c in (';', ':', '?', '!'):
            r += ';'
        elif c in ('+', '-', '*', '/', '=', '|', '_'):
            r += '-'
        elif c in ('(', '{', '[', '<'):
            r += '('
        elif c in (')', '}', ']', '>'):
            r += ')'
        else:
            r += c
    return r

def degenerate(src):
    dst = ''
    for c in src:
        if not dst or dst[-1] != c:
            dst += c
    return dst

def get_type(token):
    T = (
        'AllUpper', 'AllDigit', 'AllSymbol',
        'AllUpperDigit', 'AllUpperSymbol', 'AllDigitSymbol',
        'AllUpperDigitSymbol',
        'InitUpper',
        'AllLetter',
        'AllAlnum',
        )
    R = set(T)
    if not token:
        return 'EMPTY'

    for i in range(len(token)):
        c = token[i]
        if c.isupper():
            R.discard('AllDigit')
            R.discard('AllSymbol')
            R.discard('AllDigitSymbol')
        elif c.isdigit() or c in (',', '.'):
            R.discard('AllUpper')
            R.discard('AllSymbol')
            R.discard('AllUpperSymbol')
            R.discard('AllLetter')
        elif c.islower():
            R.discard('AllUpper')
            R.discard('AllDigit')
            R.discard('AllSymbol')
            R.discard('AllUpperDigit')
            R.discard('AllUpperSymbol')
            R.discard('AllDigitSymbol')
            R.discard('AllUpperDigitSymbol')
        else:
            R.discard('AllUpper')
            R.discard('AllDigit')
            R.discard('AllUpperDigit')
            R.discard('AllLetter')
            R.discard('AllAlnum')

        if i == 0 and not c.isupper():
            R.discard('InitUpper')

    for tag in T:
        if tag in R:
            return tag
    return 'NO'

def get_2d(token):
    return len(token) == 2 and token.isdigit()

def get_4d(token):
    return len(token) == 4 and token.isdigit()

def get_da(token):
    bd = False
    ba = False
    for c in token:
        if c.isdigit():
            bd = True
        elif c.isalpha():
            ba = True
        else:
            return False
    return bd and ba

def get_dand(token, p):
    bd = False
    bdd = False
    for c in token:
        if c.isdigit():
            bd = True
        elif c == p:
            bdd = True
        else:
            return False
    return bd and bdd

def get_all_other(token):
    for c in token:
        if c.isalnum():
            return False
    return True

def get_capperiod(token):
    return len(token) == 2 and token[0].isupper() and token[1] == '.'

def contains_upper(token):
    b = False
    for c in token:
        b |= c.isupper()
    return b

def contains_lower(token):
    b = False
    for c in token:
        b |= c.islower()
    return b

def contains_alpha(token):
    b = False
    for c in token:
        b |= c.isalpha()
    return b

def contains_digit(token):
    b = False
    for c in token:
        b |= c.isdigit()
    return b

def contains_symbol(token):
    b = False
    for c in token:
        b |= ~c.isalnum()
    return b

def b(v):
    return 'yes' if v else 'no'

def observation(v, defval=''):
    # Lowercased token.
    v['wl'] = v['w'].lower()
    # Token shape.
    v['shape'] = get_shape(v['w'])
    # Token shape degenerated.
    v['shaped'] = degenerate(v['shape'])
    # Token type.
    v['type'] = get_type(v['w'])

    # Prefixes (length between one to four).
    v['p1'] = v['w'][0] if len(v['w']) >= 1 else defval
    v['p2'] = v['w'][:2] if len(v['w']) >= 2 else defval
    v['p3'] = v['w'][:3] if len(v['w']) >= 3 else defval
    v['p4'] = v['w'][:4] if len(v['w']) >= 4 else defval

    # Suffixes (length between one to four).
    v['s1'] = v['w'][-1] if len(v['w']) >= 1 else defval
    v['s2'] = v['w'][-2:] if len(v['w']) >= 2 else defval
    v['s3'] = v['w'][-3:] if len(v['w']) >= 3 else defval
    v['s4'] = v['w'][-4:] if len(v['w']) >= 4 else defval

    # Two digits
    v['2d'] = b(get_2d(v['w']))
    # Four digits.
    v['4d'] = b(get_4d(v['w']))
    # Alphanumeric token.
    v['d&a'] = b(get_da(v['w']))
    # Digits and '-'.
    v['d&-'] = b(get_dand(v['w'], '-'))
    # Digits and '/'.
    v['d&/'] = b(get_dand(v['w'], '/'))
    # Digits and ','.
    v['d&,'] = b(get_dand(v['w'], ','))
    # Digits and '.'.
    v['d&.'] = b(get_dand(v['w'], '.'))
    # A uppercase letter followed by '.'
    v['up'] = b(get_capperiod(v['w']))

    # An initial uppercase letter.
    v['iu'] = b(v['w'] and v['w'][0].isupper())
    # All uppercase letters.
    v['au'] = b(v['w'].isupper())
    # All lowercase letters.
    v['al'] = b(v['w'].islower())
    # All digit letters.
    v['ad'] = b(v['w'].isdigit())
    # All other (non-alphanumeric) letters.
    v['ao'] = b(get_all_other(v['w']))

    # Contains a uppercase letter.
    v['cu'] = b(contains_upper(v['w']))
    # Contains a lowercase letter.
    v['cl'] = b(contains_lower(v['w']))
    # Contains a alphabet letter.
    v['ca'] = b(contains_alpha(v['w']))
    # Contains a digit.
    v['cd'] = b(contains_digit(v['w']))
    # Contains a symbol.
    v['cs'] = b(contains_symbol(v['w']))

def disjunctive(X, t, field, begin, end):
    name = '%s[%d..%d]' % (field, begin, end)
    for offset in range(begin, end+1):
        p = t + offset
        if p not in range(0, len(X)):
            continue
        X[t]['F'].append('%s=%s' % (name, X[p][field]))

U = [
    'w', 'wl', 'pos', 'chk', 'shape', 'shaped', 'type',
    'p1', 'p2', 'p3', 'p4',
    's1', 's2', 's3', 's4',
    '2d', '4d', 'd&a', 'd&-', 'd&/', 'd&,', 'd&.', 'up',
    'iu', 'au', 'al', 'ad', 'ao',
    'cu', 'cl', 'ca', 'cd', 'cs',
    ]
B = ['w', 'pos', 'chk', 'shaped', 'type']

templates = []
for name in U:
    templates += [((name, i),) for i in range(-2, 3)]
for name in B:
    templates += [((name, i), (name, i+1)) for i in range(-2, 2)]

def feature_extractor(X):
    # Append observations.
    for x in X:
        observation(x)

    # Apply the feature templates.
    crfutils.apply_templates(X, templates)

    # Append disjunctive features.
    for t in range(len(X)):
        disjunctive(X, t, 'w', -4, -1)
        disjunctive(X, t, 'w', 1, 4)

    # Append BOS and EOS features.
    if X:
        X[0]['F'].append('__BOS__')
        X[-1]['F'].append('__EOS__')

if __name__ == '__main__':
    crfutils.main(feature_extractor, fields=fields, sep=separator)



#!/usr/bin/env python

import crfsuite
import sys

# Inherit crfsuite.Trainer to implement message() function, which receives
# progress messages from a training process.
class Trainer(crfsuite.Trainer):
    def message(self, s):
        # Simply output the progress messages to STDOUT.
        sys.stdout.write(s)

def instances(fi):
    xseq = crfsuite.ItemSequence()
    yseq = crfsuite.StringList()
    
    for line in fi:
        line = line.strip('\n')
        if not line:
            # An empty line presents an end of a sequence.
            yield xseq, yseq
            xseq = crfsuite.ItemSequence()
            yseq = crfsuite.StringList()
            continue

        # Split the line with TAB characters.
        fields = line.split('\t')
        
        # Append attributes to the item.
        item = crfsuite.Item()
        for field in fields[1:]:
            p = field.rfind(':')
            if p == -1:
                # Unweighted (weight=1) attribute.
                item.append(crfsuite.Attribute(field))
            else:
                # Weighted attribute
                item.append(crfsuite.Attribute(field[:p], float(field[p+1:])))
        
        # Append the item to the item sequence.
        xseq.append(item)
        # Append the label to the label sequence.
        yseq.append(fields[0])

if __name__ == '__main__':
    # This demonstrates how to obtain the version string of CRFsuite.
    print crfsuite.version()

    # Create a Trainer object.
    trainer = Trainer()
    
    # Read training instances from STDIN, and set them to trainer.
    for xseq, yseq in instances(sys.stdin):
        trainer.append(xseq, yseq, 0)

    # Use L2-regularized SGD and 1st-order dyad features.
    trainer.select('l2sgd', 'crf1d')
    
    # This demonstrates how to list parameters and obtain their values.
    for name in trainer.params():
        print name, trainer.get(name), trainer.help(name)
    
    # Set the coefficient for L2 regularization to 0.1
    trainer.set('c2', '0.1')
    
    # Start training; the training process will invoke trainer.message()
    # to report the progress.
    trainer.train(sys.argv[1], -1)



import re
import collections

LOGDIR='log/'

def seconds(s):
    p = s.find(':')
    q = s.find(':', p+1)
    return int(s[:p]) * 3600 + int(s[p+1:q]) * 60 + int(s[q+1:])

def last(X):
    if len(X) >= 1:
        return X[-1]
    else:
        return None

def diffmin(X):
    D = []
    prev = None
    for x in X:
        if prev is not None:
            D.append(x - prev)
        prev = x
    return min(D)

def analyze_log(fi, patterns):
    P = {}
    for name, pattern, index, cast, func in patterns:
        P[name] = (re.compile(pattern), index, cast, func)

    D = collections.defaultdict(list)
    for line in fi:
        line = line.strip('\n')
        for name, (regex, index, cast, func) in P.iteritems():
            m = regex.search(line)
            if m is not None:
                if isinstance(index, tuple):
                    for i in index:
                        D[name].append(cast(m.group(i)))
                elif isinstance(index, int):
                    D[name].append(cast(m.group(index)))


    R = {}
    for name, (regex, index, cast, func) in P.iteritems():
        R[name] = func(D[name])
    return R



#!/usr/bin/env python

import re
import sys

class FeatureExtractor:
    def __init__(self):
        self.macro = re.compile(r'%x\[(?P<row>[\d-]+),(?P<col>[\d]+)\]')
        self.inst = []
        self.t = 0
        self.templates = []

    def read(self, fi):
        self.templates = []
        for line in fi:
            line = line.strip()
            if line.startswith('#'):
                continue
            if line.startswith('U'):
                self.templates.append(line.replace(':', '='))
            elif line == 'B':
                continue
            elif line.startswith('B'):
                sys.stderr(
                    'ERROR: bigram templates not supported: %s\n' % line)
                sys.exit(1)

    def replace(self, m):
        row = self.t + int(m.group('row'))
        col = int(m.group('col'))
        if row in range(0, len(self.inst)):
            return self.inst[row]['x'][col]
        else:
            return ''

    def apply(self, inst, t):
	self.inst = inst
	self.t = t
        for template in self.templates:
            f = re.sub(self.macro, self.replace, template)
            self.inst[t]['F'].append(f)

def readiter(fi, sep=None):
    X = []
    for line in fi:
        line = line.strip('\n')
        if not line:
            yield X
            X = []
        else:
            fields = line.split(sep)
            item = {
                'x': fields[0:-1],
                'y': fields[-1],
                'F': []
                }
            X.append(item)

if __name__ == '__main__':
    import optparse

    fi = sys.stdin
    fo = sys.stdout

    # Parse the command-line arguments.
    parser = optparse.OptionParser(usage="""usage: %prog <template>
This utility reads a data set from STDIN, applies feature templates compatible
with CRF++, and outputs attributes to STDOUT. Each line of a data set must
consist of field values separated by SEPARATOR characters (customizable with
-s option)."""
        )
    parser.add_option(
        '-s', dest='separator', default='\t',
        help='specify the separator of columns of input data [default: "\\t"]'
        )
    (options, args) = parser.parse_args()

    F = FeatureExtractor()
    F.read(open(args[0]))

    for inst in readiter(fi, options.separator):
        for t in range(len(inst)):
            F.apply(inst, t)
            fo.write('%s' % inst[t]['y'])
            for attr in inst[t]['F']:
                fo.write('\t%s' % attr.replace(':', '__COLON__'))
            fo.write('\n')
        fo.write('\n')
